# Copyright (c) 2016, Patrick Uiterwijk <patrick@puiterwijk.org>
# All rights reserved.
#
# This file is part of Basset.
#
# Basset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Basset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Basset.  If not, see <http://www.gnu.org/licenses/>.

from basset.score.base import ScorePlugin

import requests


class ContentScore(ScorePlugin):
    name = 'content'

    def get_score(self, action, time, data):
        scores = {}

        title = None
        text = None
        comment = None

        if 'page' in data:
            page = data['page']
            if 'title' in page:
                title = page['title']
            elif 'name' in page:
                title = page['name']
            if 'text' in page:
                text = page['text']
            if 'comment' in page:
                comment = page['comment']
            if 'revisionurl' in page:
                text = requests.get(page['revisionurl']).text
        elif 'ticket' in data:
            title = data['ticket']['summary']
            text = data['ticket']['description']
            comment = data.get('comment')
        elif 'attachment' in data:
            title = data['attachment']['filename']
            text = data['attachment']['description']
        else:
            self.log.info('No page info found')


        if title:
            scores['title'] = self._get_title_score(title)
        else:
            self.log.info('No title detected')

        if text:
            scores['text'] = self._get_title_score(text)
        else:
            self.log.info('No text detected')

        if comment:
            scores['comment'] = self._get_title_score(comment)
        else:
            self.log.info('No comment detected')

        return scores

    def _get_title_score(self, title):
        replacements = [
            ('+', ' '),
            ('%20', ' '),
            (' ', '_'),
            ('-', ''),
            ('.', ''),
        ]
        for replacement in replacements:
            title = title.replace(*replacement)
        title = title.lower()
        mapping = {}
        for option in self.config:
            if option.startswith('title_'):
                key = option[len('title_'):]
                for replacement in replacements:
                    key = key.replace(*replacement)
                key = key.lower()
                mapping[key] = self.config[option]

        total_score = 0
        for word in mapping:
            if word in title:
                total_score += int(mapping[word])
        return total_score
