# Copyright (c) 2016, Patrick Uiterwijk <patrick@puiterwijk.org>
# All rights reserved.
#
# This file is part of Basset.
#
# Basset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Basset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Basset.  If not, see <http://www.gnu.org/licenses/>.

import dns.name
import dns.resolver
import dns.reversename

from basset.score.base import ScorePlugin


class MailExchangeScore(ScorePlugin):
    name = 'mailexchange'
    quick = False

    def _do_init(self):
        # Initialize
        self._evilsuperdomains = {}
        for option in self.config:
            if option.startswith('superdomain_'):
                key = option[len('superdomain_'):]
                self._evilsuperdomains[key] = self.config[option]

    def get_score(self, action, time, data):
        email = data['user']['email']

        scores = {}
        scores['mx'] = self.get_mx_score(email, False)
        scores['mx_triple'] = self.get_mx_score(email, True)

        return scores

    def get_mx_score(self, email, do_triple):
        username, domain = email.split('@')
        try:
            answers = dns.resolver.query(domain, 'MX')
        except Exception as ex:
            self.log.error('Unable to get mail servers: %s' % ex)
            return 10
        score = 0
        if do_triple:
            todo = list(answers)
            answers = []
            for mx in todo:
                result = dns.resolver.query(mx.exchange, 'A')
                for item in result.rrset.items:
                    rev = dns.reversename.from_address(item.address)
                    for rdns in dns.resolver.query(rev, 'PTR').rrset.items:
                        answers.append(rdns.target)
        else:
            answers = [answer.exchange for answer in answers]

        for answer in answers:
            for evildomain in self._evilsuperdomains:
                dmn = dns.name.from_text(evildomain)
                if answer.is_subdomain(dmn):
                    score += int(self._evilsuperdomains[evildomain])
        return score
